#
# (C) Tenable Network Security
# 


if (description) {
  script_id(18644);
  script_version("$Revision: 1.3 $");

  script_cve_id("CAN-2005-2215");
  script_bugtraq_id(14181);
  if (defined_func("script_xref")) {
    script_xref(name:"OSVDB", value:"17763");
  }

  name["english"] = "MediaWiki Page Move Template Cross-Site Scripting Vulnerability";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote host is running MediaWiki, a wiki web application written
in PHP. 

The remote version of this software, according to its version number,
is vulnerable to cross-site scripting attacks because of its failure 
to sanitize input passed to the page move template.  This flaw could 
be used to inject arbitrary HTML and script code into a user's browser 
resulting in the theft of cookies, misrepresentation of the site, and 
other such attacks. 

See also : http://bugzilla.wikimedia.org/show_bug.cgi?id=2304
Solution : If using MediaWiki 1.4.x, upgrade to 1.4.6 or later;
           if using MediaWiki 1.5.x, upgrade to 1.5.0 beta3 or later.
Risk factor : Low";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for page move template cross-site scripting vulnerability in MediaWiki";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_family(english:"CGI abuses : XSS");

  script_copyright(english:"This script is Copyright (C) 2005 Tenable Network Security");

  script_dependencie("http_version.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}


include("http_func.inc");
include("http_keepalive.inc");


port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);
if (!can_host_php(port:port)) exit(0);


# For each CGI directory...
foreach dir (cgi_dirs()) {
  # Try to get MediaWiki's version number.
  req = http_get(item:string(dir, "/index.php?title=Special:Version"), port:port);
  res = http_keepalive_send_recv(port:port, data:req, bodyonly:FALSE);
  if (res == NULL) exit(0);

  # If it looks like MediaWiki, grab its version number.
  #
  # nb: this doesn't catch the really old versions (MediaWiki-stable 
  #     20031117 and older), but they no longer appear to be deployed.
  pat = ">MediaWiki</a>.+: ([0-9]+\.[0-9]+.*)";
  if (egrep(string:res, pattern:pat, icase:TRUE)) {
    matches = egrep(pattern:pat, string:res, icase:TRUE);
    foreach match (split(matches)) {
      match = chomp(match);
      ver = eregmatch(pattern:pat, string:match);
      if (!isnull(ver)) {
        ver = ver[1];
        iver = split(ver, sep:'.', keep:FALSE);

        # Check for a vulnerable version.
        if (ver =~ "^1\.(4\.(0 beta6|[1-5])|5\.0 (alpha|beta[12]))") {
          security_warning(port);
          exit(0);
        }

        # Must not have been vulnerable -- we're done with the matches.
        break;
      }
    }
  }
}
